@@ -1,4 +1,5 @@
-# Copyright (c) 2018 The Monero Project
+# Copyright (c) 2018-2022, The Monero Project
+
 # 
 # All rights reserved.
 # 
@@ -53,6 +54,41 @@ class Daemon(object):
         return self.rpc.send_json_rpc_request(getblocktemplate)
     get_block_template = getblocktemplate
 
+    def get_miner_data(self):
+        get_miner_data = {
+            'method': 'get_miner_data',
+            'jsonrpc': '2.0', 
+            'id': '0'
+        }
+        return self.rpc.send_json_rpc_request(get_miner_data)
+
+    def calc_pow(self, major_version, height, block_blob, seed_hash = ''):
+        calc_pow = {
+            'method': 'calc_pow',
+            'params': {
+                'major_version': major_version,
+                'height': height,
+                'block_blob' : block_blob,
+                'seed_hash' : seed_hash,
+            },
+            'jsonrpc': '2.0',
+            'id': '0'
+        }
+        return self.rpc.send_json_rpc_request(calc_pow)
+
+    def add_aux_pow(self, blocktemplate_blob, aux_pow, client = ""):
+        add_aux_pow = {
+            'method': 'add_aux_pow',
+            'params': {
+                'blocktemplate_blob': blocktemplate_blob,
+                'aux_pow' : aux_pow,
+                'client' : client,
+            },
+            'jsonrpc': '2.0',
+            'id': '0'
+        }
+        return self.rpc.send_json_rpc_request(add_aux_pow)
+
     def send_raw_transaction(self, tx_as_hex, do_not_relay = False, do_sanity_checks = True, client = ""):
         send_raw_transaction = {
             'client': client,
